import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { CatIndicesIndicesRecord, CatAliasesAliasesRecord } from '@elastic/elasticsearch/lib/api/types';

@Injectable({ providedIn: 'root' })
export class IndexManagementService {
  resourceUrl = this.applicationConfigService.getEndpointFor('api/esManagement');

  constructor(private http: HttpClient, private applicationConfigService: ApplicationConfigService) {}

  getIndices(): Observable<CatIndicesIndicesRecord[]> {
    return this.http.get<CatIndicesIndicesRecord[]>(this.resourceUrl + '/indices');
  }

  getAliasses(): Observable<CatAliasesAliasesRecord[]> {
    return this.http.get<CatAliasesAliasesRecord[]>(this.resourceUrl + '/aliases');
  }

  deleteIndex(index: string): Observable<boolean> {
    return this.http.delete<boolean>(this.resourceUrl + '/index/' + index);
  }

  reindex(): Observable<string> {
    const opts: any = {
      headers: new HttpHeaders({
        Accept: 'text/html, application/xhtml+xml, */*',
        'Content-Type': 'text/plain; charset=utf-8',
      }),
      responseType: 'text',
    };
    return this.http.post<string>(this.resourceUrl + '/reindex', opts);
  }

  getLog(id: string): Observable<LoggerInfo> {
    return this.http.get<LoggerInfo>(this.resourceUrl + '/reindexLog/' + id);
  }
}

export interface LoggerInfo {
  log: string;
  closed: boolean;
}
