import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, ActivatedRoute, Router, convertToParamMap } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { of } from 'rxjs';

import { IUserWatchList, UserWatchList } from '../user-watch-list.model';
import { UserWatchListService } from '../service/user-watch-list.service';

import { UserWatchListRoutingResolveService } from './user-watch-list-routing-resolve.service';

describe('UserWatchList routing resolve service', () => {
  let mockRouter: Router;
  let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
  let routingResolveService: UserWatchListRoutingResolveService;
  let service: UserWatchListService;
  let resultUserWatchList: IUserWatchList | undefined;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes([])],
      providers: [
        {
          provide: ActivatedRoute,
          useValue: {
            snapshot: {
              paramMap: convertToParamMap({}),
            },
          },
        },
      ],
    });
    mockRouter = TestBed.inject(Router);
    jest.spyOn(mockRouter, 'navigate').mockImplementation(() => Promise.resolve(true));
    mockActivatedRouteSnapshot = TestBed.inject(ActivatedRoute).snapshot;
    routingResolveService = TestBed.inject(UserWatchListRoutingResolveService);
    service = TestBed.inject(UserWatchListService);
    resultUserWatchList = undefined;
  });

  describe('resolve', () => {
    it('should return IUserWatchList returned by find', () => {
      // GIVEN
      service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultUserWatchList = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultUserWatchList).toEqual({ id: 123 });
    });

    it('should return new IUserWatchList if id is not provided', () => {
      // GIVEN
      service.find = jest.fn();
      mockActivatedRouteSnapshot.params = {};

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultUserWatchList = result;
      });

      // THEN
      expect(service.find).not.toBeCalled();
      expect(resultUserWatchList).toEqual(new UserWatchList());
    });

    it('should route to 404 page if data not found in server', () => {
      // GIVEN
      jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as UserWatchList })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultUserWatchList = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultUserWatchList).toEqual(undefined);
      expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
    });
  });
});
